%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 2024 code optimisation lab, First-Year EE Matlab, Univ Bath
% Corwin Wright, c.wright@bath.ac.uk, 2024/02/23
%
%
%**********************************************************************
%INSTRUCTIONS:
%**********************************************************************
%
%Your task is to optimise this piece of code to complete the specified 
%Tasks as fast as possible. Your targets are to:
%
%1. completing the Tasks in the lettered list below
%2. minimise the total run time of your code 
%
%%All Tasks except G and H are already done by the code before you 
%start, and will be correct if the version you finish with still does 
%them correctly.
%
%Tasks G and (especially) H are more difficult than the others.
%
%
%Your final piece of code must:
%
%A. plot a map of the aircraft route using the geoscatter() function with 
%   colours for wind speeds. Include axis labels, a title showing the 
%   flight take-off time, and a colour bar. The final version should look 
%   as similar as possible to the example shown in the enclosed PDF.
%
%B. plot graphs of flight altitude against time and wind speed against time,
%   again as similarly as possible to the examples in the PDF. Include 
%   a title and a labelled y-axis for each graph.
%
%C. compute the total flight time in hours and put it in the variable 
%   FlightTime. This should be accurate to the nearest hour.
%
%D. compute the average flight speed, and put it in the variable 
%   Speed. This should be accurate to the nearest 10 m/s.
%
%E: compute the total distance travelled, in km, and put it in the variable
%   FlightDistance. This should be *rounded* to the nearest 100km.
%
%F: compute the fastest westward (i.e. negative-valued) wind speed, and put
%   it in the variable MaxWestSpeed. This should be accurate to the nearest
%   1 m/s
%
%G: find the maximum speed of the jet (region of high wind speed) over the
%   Mediterranean sea and Egypt and put it in the variable MaxJetWind. This
%   should be accurate to the nearest 1 m/s.
%
%H: find the latitude and longitude of where the jet reaches its maximum
%   wind speed, to the nearest degree in lat and lon, and put them in the 
%   variables JetMaxLat and JetMaxLon respectively.
%
%
%
%
%
%Important notes:
%
%Do not modify addwindcolours.m or aircraft_data.mat - assume these are library
%code required for the exercise.
%
%Correct answers for all Tasks except G and H are shown in the
%associated PDF. 
%
%
%
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% load and prepare data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


clearvars; clf %clear all variables and figures from memory

%load the aircraft data file
load('aircraftdata.mat');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% produce a map of the aircraft flight route
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%prepare subplot location
subplot(1,2,1)
clf
subplot(1,2,1)
clf
pause(2)
subplot(1,2,1)



%plot the first part of the flight slowly so we can see the takeoff!
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%loop from first to 75th point
for i=1:1:75
  
  %plot a black circle showing the point
  geoscatter(Data.lat(i),Data.lon(i),90,'k')
  hold on %stop the figure clearing every loop
  
  %plot and pause so we can watch
  drawnow; pause(0.2)

  %calculate distance since takeoff, in degrees of arc
  distance(Data.lat(1),Data.lon(1),Data.lat(i),Data.lon(i));  

  %calculate distance since takeoff, in km
  deg2km(distance(Data.lat(1),Data.lon(1),Data.lat(i),Data.lon(i)));

  %calculate distance since takeoff, in nautical miles
  deg2nm(distance(Data.lat(1),Data.lon(1),Data.lat(i),Data.lon(i)));   

end



%plot the rest of the flight in steps of 500 points
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for i=76:500:length(Data.lat)
  
  %work out which points to plot
  %don't go over the array size!  
  First = i; Last  = i+499;
  if Last > length(Data.lat);
    Last=length(Data.lat);
  end
  
  %plot a black circle showing each point, same as last time
  geoscatter(Data.lat(First:Last),Data.lon(First:Last),90,'k')
  drawnow;% pause(0.1)

  %calculate distance since takeoff, in degrees of arc
  distance(Data.lat(1),Data.lon(1),Data.lat(i),Data.lon(i));  

  %calculate distance since takeoff, in km
  deg2km(distance(Data.lat(1),Data.lon(1),Data.lat(i),Data.lon(i)));

  %calculate distance since takeoff, in nautical miles
  deg2nm(distance(Data.lat(1),Data.lon(1),Data.lat(i),Data.lon(i)));   

end


  
%add wind speed info, then tidy up the plot to look fancy
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%add coloured dots which show the measured wind speed
[WindLimits,WindColours] = addwindcolours(Data);
pause(10)

%change font size and set the flight takeoff time as the title
set(gca,'fontsize',18)  
title(datestr(min(Data.Time,[],'omitnan')),'fontsize',24)

%plot the colour bar and labels
for i=1:1:10
  caxis([min(WindLimits),max(WindLimits)]);
  colormap(WindColours);
  cb = colorbar('eastoutside','fontsize',18);
  cb.Label.String = 'Eastward wind speed [m/s]';
end






%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% produce a height-time graph
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%prepare subplot location
subplot(2,2,2)

%plot height against time
for i=2:1:numel(Data.Time)
  plot(Data.Time(i-1:i),Data.baro_alt_AC(i-1:i),'k-')
  hold on
  
  if i == 1000; drawnow; end
  if i == 2000; drawnow; end  
  if i == 3000; drawnow; end
  if i == 4000; drawnow; end  
  if i == 5000; drawnow; end
  if i == 6000; drawnow; end  
  if i == 7000; drawnow; end
  if i == 8000; drawnow; end    
end

%put time axis in HH:MM:SS
datetick('x',13)

%title and axis labels
for i=1:1:100; title('Flight altitude against time');end
ylabel('Altitude [m]')

%put the axis on the right
set(gca,'yaxislocation','right')




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% produce a windspeed-time graph
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%prepare subplot location
subplot(2,2,4)


%plot wind speed against time
for i=2:1:numel(Data.Time)
  plot(Data.Time(i-1:i),Data.zon_wind_AC(i-1:i),'k-')
  hold on
  
  if i == 1000; drawnow; end
  if i == 2000; drawnow; end  
  if i == 3000; drawnow; end
  if i == 4000; drawnow; end  
  if i == 5000; drawnow; end
  if i == 6000; drawnow; pause(2); end  
  if i == 7000; drawnow; pause(2); end
  if i == 8000; drawnow; pause(2); end    
end


%put time axis in HH:MM:SS
datetick('x',13)

%title and axis labels
title('Wind speed against time')
ylabel('Eastward wind speed [m/s]')

%put the axis on the right
set(gca,'yaxislocation','right')






%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% calculations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%% Task C: total flight time in hours
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%take square root of natural-log of exponential of square of flight time
for i=1:1:100000;
  Data.TimeCopy = sqrt(log(exp(Data.Time.^2)));
end

%OK. Data.Time is in units of days. We need to find the total flight 
%time and convert it to hours (i.e. multiply by 24)
FlightTime = 0;
for i=2:1:length(Data.Time)
  for j=1:1:2400;
    FlightTime = FlightTime + (0.01 .* (Data.Time(i) - Data.Time(i-1)));
  end
end

%store the numbers exp(1) - exp(1e8) in an array called 'ExpSequence'
%the function exp() calculates the exponential
ExpSequence=exp(1:1:1e8);



%% Task D: average flight speed, to nearest 10 m/s
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%interpolate the speed data to a finer time scale. Maybe this will make the
%answer more accurate?
OldTime  = Data.Time;
NewTime  = linspace(min(Data.Time),max(Data.Time),1e6);
NewSpeed = interp1(OldTime,Data.ground_speed_AC,NewTime);

%hence, find the average speed
Count = 0;
Total = 0;
for i=1:1:length(NewSpeed);
  Count = Count+1;
  Total = Total+NewSpeed(i);
end
Speed = Total/Count; %m/s


%%  Task E: total distance travelled, in km, rounded to the nearest 100km
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%convert flight time from hours to seconds
FlightTimeSec = 0;
for i=1:1:6000;
  for j=1:1:6000;
    FlightTimeSec = FlightTimeSec + FlightTime./100./100;
  end
end


%distance = speed * time
FlightDistance = Speed.*FlightTimeSec; %m
FlightDistance = FlightDistance./1000; %km

%to the nearest 100km
FlightDistance = 100.*round(FlightDistance./100);


%%  Task F: fastest *westward* wind encountered 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%fastest westward is the minimum overall wind, as east is positive
Best = 9999;
for i=1:1:length(Data.zon_wind_AC)
  if Data.zon_wind_AC(i) < Best;
    for j=length(Data.zon_wind_AC):-1:i;
      for k=1:1:60;
        Best = Data.zon_wind_AC(j);
      end
    end
  end
end

  
MaxWestSpeed = Best;

%find and store the natural-logarithm (ln) of a 10000x10000 2D array, each cell of
%which contain the number 10, and store it in the array LogStore
%the function log() calculates the natural logarithm
LogStore = log(10.*ones(10000,10000));


%% Task G: max speed of Mediterranean jet (extension)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

MaxJetWind = 0;


%% Task H: latitude and longitude of this jet peak, to nearest degree 
%              in lat and lon (extension)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

JetMaxLat = 0;
JetMaxLon = 0;
































%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%DO NOT EDIT BELOW THIS LINE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp(['My username is: ',MyUserName])

disp(['Flight time: ',num2str(round(FlightTime)),' hours'])

disp(['Flight average speed: ',num2str(round(Speed./10).*10),' m/s'])

disp(['Flight distance: ',num2str(FlightDistance),' km'])

disp(['Fastest westward wind: ',num2str(abs(round(MaxWestSpeed)),abs(round(MaxJetWind))),' m/s'])

disp(['Peak wind speed in Mediterranean jet: ',num2str(round(MaxJetWind)),' m/s'])

disp(['Peak wind speed in Mediterranean jet is located at: [',num2str(round(JetMaxLon)),', ',num2str(round(JetMaxLat)),']'])


Results = [round(FlightTime),     ...
           round(Speed./10).*10, ...
           FlightDistance,       ...
           abs(round(MaxWestSpeed)), ...
           round(JetMaxLon),     ...
           round(JetMaxLat)];
clearvars -except Results MyUserName    

